///////////////////////////////////////////////////////////////////////////////////////////////////////
// Global Variables
///////////////////////////////////////////////////////////////////////////////////////////////////////

// Time-keeping

global int _playbackActive;

global float _bpm;
((60.0 / _bpm ) * 4.0 * 1000.0)::ms => dur BEAT;
//BEAT - (now % BEAT) => now;
// update rate for pos
10::ms => dur POS_RATE;
// increment per step
POS_RATE/BEAT => global float posInc;

// outgoing variables sent to Unity
global float _position;

global int _currentV0; // wish I could just sync arrays w/ Chunity :(
global int _currentV1;
global int _currentV2;
global int _currentV3;
global int _currentV4;
global int _currentV5;
global int _currentV6;
global int _currentV7;


// for selecting and editing whole polygons

global int _whichPoly; // index of currently selected polygon 
global int _N; // size of currently selected polygon


// for selecting and editing individual vertices
global int _whichV; // index of currently selected vertex of selected polygon
global float _editRate; // playback rate for selected vertex
global float _editGain; // gain for selected vertex


// events for communication with Unity

global Event changeN; // signal from Unity to change which Polygon is selected
global Event editVertex; // signal from Unity to update the size of currently selected polygon (NUM_VERTICES)
global Event changeBPM; // signal from Unity to change the tempo
global Event togglePlayback;

///////////////////////////////////////////////////////////////////////////////////////////////////////
// Polygon Class
///////////////////////////////////////////////////////////////////////////////////////////////////////
class Polygon
{
    // pre-construction
    
    8 => int num_vertices;
    0 => int currentVertex;
    0.0 => float position;
    float seqRate[num_vertices];
    float seqGain[num_vertices];
    
    SndBuf bufs[num_vertices];
    NRev reverb => dac;
    .1 => reverb.mix;
    
    for( int i; i < num_vertices; i++ )
    {
        .1 => seqGain[i];
        1 => seqRate[i];

        // connect to dac
        bufs[i] => reverb;
        // load sound
        me.dir() + "data/ping.wav" => bufs[i].read;
        // silence
        0 => bufs[i].gain;
    }
    fun void reset(int new_N, int multiplier) // this gets called if the user changes N for a particular polygon. It should overwrite the initialized variables (even new array sizes!)
    {
        
        new_N => num_vertices;
        float new_seqRate[new_N];
        float new_seqGain[new_N];
        SndBuf new_bufs[new_N];
        
        for( int i; i < new_N; i++ )
        {
            .1 => new_seqGain[i];
            1.0+(multiplier*.25) => new_seqRate[i];
            
            // connect to dac
            new_bufs[i] => reverb;
            // load sound
            me.dir() + "data/ping.wav" => new_bufs[i].read; // +++ eventually, allow for different samples across vertices
            // silence
            0 => new_bufs[i].gain;
        }
        
        // Actual reassignment
        new_seqRate @=> seqRate;
        new_seqGain @=> seqGain;
        new_bufs @=> bufs;
    }
    fun void silence()
    {
        for( int i; i < num_vertices; i++ )
        {
            0.0 => seqGain[i];
        }
    }
    fun void play() // this runs for exactly one measure, regardless of subdivision
    {
        if ( num_vertices != 0)
        {
            
            for(0 => int i; i< num_vertices; i++)
            {
                // Play current vertex
                // NOTE: currentVertex and position belong to one Polygon instance, and are NOT globals
                
                // start from beginning of clip
                0 => bufs[currentVertex].pos;
                // set gain
                seqGain[currentVertex] => bufs[currentVertex].gain;
                // set rate
                seqRate[currentVertex] => bufs[currentVertex].rate;
                
                // sync with discrete grid position
                //(_position/4.0) * num_vertices) => position;
                currentVertex => position;
                //currentVertex * (4.0 / num_vertices) => _position;
                // advance time by duration of one beat subdivided according to the size of the polygon
                //BEAT / num_vertices => now;
                ((60.0 / _bpm ) * 4.0*1000.0)::ms => BEAT;
                (BEAT/num_vertices) - (now % (BEAT/num_vertices)) => now;
                
                // increment to next vertex
                currentVertex++;
                // wrap
                if( currentVertex >= num_vertices ) 
                {
                    0 => currentVertex;
                }         
            } 
        }


    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
// Local variables
///////////////////////////////////////////////////////////////////////////////////////////////////////

int seq_PolyN[8]; // holds N values for each polygon in the ring (hard-coded max size = 8) 
Polygon @ allPolygons[8]; // the array of all Polygon instances
//new Polygon @=> allPolygons[0];
//allPolygons[0].reset(2,0);
for(0 => int i; i < 8; i++)
{
    new Polygon @=> allPolygons[i];
    allPolygons[i].reset(8, i);
    allPolygons[i].silence();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions
///////////////////////////////////////////////////////////////////////////////////////////////////////

// fine-grained update function to advance time and sync playback to grid
// NOTE: separate position variable tracked for each individual polygon
fun void updatePositions()
{
    while( true )
    {
        for( 0 => int i; i < 8; i++)
        {   
            // increment
            //posInc +=> allPolygons[i].position;
            (posInc*allPolygons[i].num_vertices) +=> allPolygons[i].position;
            //4.0*allPolygons[i].position => _position;
            if(allPolygons[i].num_vertices !=0)
            {
                4.0*allPolygons[i].position / allPolygons[i].num_vertices => _position;
            }
        }
        
        posInc +=> _position;
        //<<<_position>>>;
        // advance time
        POS_RATE => now;
    }
}
/*
fun void syncTrack()
{
    while(true)
    {
        if(_position >= 4.0)
        {
            0.0 => _position;
        }
        posInc +=> _position;
        
        POS_RATE => now;
    }
}
*/
/*
fun void syncPositions()
{
    while(true)
    {
        if(allPolygons[0].position == 0.0)
        {
            for(1 => int i; i<8; i++)
            {
                0.0 => allPolygons[i].position;
            }
        }
    }
}
*/
/*
fun void listen_changeBPM()
{
    while(true)
    {
        changeBPM => now; //maybe go back to using an event from unity?
        
        _bpm => _bpm;
        //<<<_bpm>>>;
        ((60.0 / _bpm ) * 4.0*1000.0)::ms => BEAT;
        //BEAT - (now % BEAT) => now;
        // increment per step
        POS_RATE/BEAT => posInc;
    }
}
*/
// listen for signal to change N value for the selected polygon
fun void listen_changeN() 
{
    while(true)
    {
        changeN => now; // changePoly event from Unity needs to also send _whichPoly and _N
        
        if(_N == 0)
        {
            allPolygons[_whichPoly].silence();
        }
        else
        {
            //BEAT - (_position*BEAT*.25) => now;
            allPolygons[_whichPoly].reset(_N, _whichPoly); // calls class function 'reset' which re-initializes the polygon using new N value
            <<< "Polygon #",_whichPoly, "changed size to: ", _N >>>;
        }
        
    }
}

// listen for signal to change the rate or gain of a selected vertex
fun void listen_editV()
{
    while( true )
    {
        // wait for event
        editVertex => now;
        // update the gain and rate; NEEDS _whichPoly, _whichV, _editRate, and _editGain from Unity
        _editRate => allPolygons[_whichPoly].seqRate[_whichV]; // NOT SURE THIS WILL WORK - test in scratch
        _editGain => allPolygons[_whichPoly].seqGain[_whichV]; // NOT SURE THIS WILL WORK - test in scratch
        <<< "Vertex #",_whichV," of Polygon #",_whichPoly,"changed... rate:", _editRate, "gain:", _editGain >>>;
    }
}

// to send to Unity--tells the index of the current Vertex playing, AND the position (within each of the 8 polygons)
fun void updateCurrentVP()
{
    while( true )
    {
        allPolygons[0].currentVertex => _currentV0;
        allPolygons[1].currentVertex => _currentV1;
        allPolygons[2].currentVertex => _currentV2;
        allPolygons[3].currentVertex => _currentV3;
        allPolygons[4].currentVertex => _currentV4;
        allPolygons[5].currentVertex => _currentV5;
        allPolygons[6].currentVertex => _currentV6;
        allPolygons[7].currentVertex => _currentV7;
        
        //<<<"Global position: ", _position>>>;
        POS_RATE => now;
    }
}
/*
fun void startAllPoly() // This needs to be added to?
{

    for( 0 => int i; i < 8; i++)
    {
        allPolygons[i].play();
    }

}
*/
fun void loopPlayPoly(int which)
{
    while(true)
    { 
        allPolygons[which].play();
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////////
// Execution and control
///////////////////////////////////////////////////////////////////////////////////////////////////////

//spork ~ listen_changeBPM();
spork ~ listen_changeN(); // listener for Unity signals to edit N
spork ~ listen_editV(); // listener for Unity signals to edit a vertex
spork ~ updatePositions(); // time-keeping, must always be running to ensure sync
spork ~ updateCurrentVP(); // gets info continuously from each Polygon instance to send to Unity 
//spork ~ syncPositions();
//spork ~ startAllPoly(); // one-time spork to start all the Polygons // +++ eventually, add ability to pause/stop individual Polygons
spork ~ loopPlayPoly(0);
spork ~ loopPlayPoly(1);
spork ~ loopPlayPoly(2);
spork ~ loopPlayPoly(3);
spork ~ loopPlayPoly(4);
spork ~ loopPlayPoly(5);
spork ~ loopPlayPoly(6);
spork ~ loopPlayPoly(7);
// TO DO
// need to accommodate multiple polygons. Need to set/initialize each one, not just the initially selected one
// need to keep track of all N values, as well as all rate/gain values. Need to do multi/jagged arrays like in Unity



// parent shred to keep child shreds running
while(true)
{
    BEAT - (now % BEAT) => now;
}

/*
while (true)
{
    togglePlayback => now;
    
    if(_playbackActive == 1)
    {
        while(true)
        {
            1::second => now;
        }
    }

}
*/