// number of vertices (MUST match the Unity NUM_VERTICES!)
//4 => int NUM_VERTICES;
//BPM
120 => float bpm;
((60.0 / bpm ) * 1000.0)::ms => dur BEAT;
// update rate for pos
10::ms => dur POS_RATE;
// increment per step
POS_RATE/BEAT => float posInc;

// global (outgoing) variables for Unity animation
global int currentVertex; // which vertex we are at
// will take on fractional values for smooth animation[0-NUM_VERTICES)
global float playheadPos;

// global (incoming) data from Unity
4 => global int num_vertices;
global int editWhich;
1 => global float editRate;
1 => global float editGain;
global Event editHappened;
//num_vertices => int NUM_VERTICES; // ALL CAPS is ChucK variable

// a sequence (one of many ways to "sequence")
// this ones treats vertices as a step sequencer
float seqRate[num_vertices];
float seqGain[num_vertices];
// initialize

for( int i; i < num_vertices; i++ )
{
    1 => seqGain[i];
    1 => seqRate[i];
}

// sound buffers
SndBuf bufs[num_vertices];
// reverb
NRev reverb => dac;
// reverb mix
.1 => reverb.mix;

// connect them
for( int i; i < num_vertices; i++ )
{
    // connect to dac
    bufs[i] => reverb;
    // load sound
    me.dir() + "data/snare.wav" => bufs[i].read;
    // silence
    0 => bufs[i].gain;
}

// spork update
spork ~ playheadPosUpdate();
// spork edit listener
spork ~ listenForEdit();

// simple sequencer loop
while( true )
{
    // play current
    //currentVertex => float currentVertexFloat;
    //(playheadPos % 4) $ int => int playHeadPosInt;
    //if(currentVertex == playHeadPosInt)
    //{
    //    <<<"bang!">>>; current Vertex should light up! (in unity)
    //}

    play( currentVertex,
          seqGain[currentVertex], seqRate[currentVertex] );
    // sync with discrete grid position
    currentVertex => playheadPos;
    // advance time by duration of one beat 
    BEAT => now;
    // increment to next chicken
    currentVertex++;
    // wrap
    if( currentVertex >= num_vertices ) 0 => currentVertex;
}

fun void play( int which, float gain, float rate )
{
    // restart
    0 => bufs[which].pos;
    // set gain
    gain => bufs[which].gain;
    // set rate
    rate => bufs[which].rate;
}

// updates the global playheadPos with fine granularity,
// for visualizing the playhead smoothly in Unity
fun void playheadPosUpdate()
{
    while( true )
    {
        // increment
        posInc +=> playheadPos;
        //<<<playheadPos>>>;
        // advance time
        POS_RATE => now;
    }
}


// this listens for events from Unity to update the sequence
fun void listenForEdit()
{
    while( true )
    {
        // wait for event
        editHappened => now;
        // update the gain and rate
        editRate => seqRate[editWhich];
        editGain => seqGain[editWhich];
        <<< num_vertices >>>;
        <<< "EDIT rate:", editRate, "gain:", editGain >>>;
    }
}