// number of vertices (MUST match the Unity NUM_VERTICES!)
//4 => int NUM_VERTICES;
//BPM
120 => float bpm;
((60.0 / bpm ) * 1000.0)::ms => dur BEAT;
// update rate for pos
10::ms => dur POS_RATE;
// increment per step
POS_RATE/BEAT => float posInc;

// global (outgoing) variables for Unity animation
global int currentVertex; // which vertex we are at
// will take on fractional values for smooth animation[0-NUM_VERTICES)
global float playheadPos;

// global (incoming) data from Unity
4 => global int NUM_VERTICES;
global int editWhich;
1 => global float editRate;
1 => global float editGain;
global Event editHappened;
global Event nChange;

NUM_VERTICES => int local_N;

fun void set_and_run(int num_vertices)
{
    // a sequence (one of many ways to "sequence")
    // this ones treats vertices as a step sequencer
    float seqRate[num_vertices];
    float seqGain[num_vertices];
    // initialize

    for( int i; i < num_vertices; i++ )
    {
        1 => seqGain[i];
        1 => seqRate[i];
    }

    // sound buffers
    SndBuf bufs[num_vertices];
    // reverb
    NRev reverb => dac;
    // reverb mix
    .1 => reverb.mix;

    // connect them
    for( int i; i < num_vertices; i++ )
    {
        // connect to dac
        bufs[i] => reverb;
        // load sound
        me.dir() + "data/snare.wav" => bufs[i].read;
        // silence
        0 => bufs[i].gain;
    }

    // simple sequencer loop
    while( true )
    {         
        // PLAY CURRENT VERTEX  
                
        // start from beginning of clip
        0 => bufs[currentVertex].pos;
        // set gain
        seqGain[currentVertex] => bufs[currentVertex].gain;
        // set rate
        seqRate[currentVertex] => bufs[currentVertex].rate;
        // sync with discrete grid position
        currentVertex => playheadPos;
        // advance time by duration of one beat 
        BEAT => now;
        // increment to next chicken
        currentVertex++;
        // wrap
        if( currentVertex >= num_vertices ) 0 => currentVertex;
    }  
}


// updates the global playheadPos with fine granularity,
// for visualizing the playhead smoothly in Unity
fun void playheadPosUpdate()
{
    while( true )
    {
        // increment
        posInc +=> playheadPos;
        //<<<playheadPos>>>;
        // advance time
        POS_RATE => now;
    }
}

/*
// this listens for events from Unity to update the sequence
fun void listenForEdit()
{
    while( true )
    {
        // wait for event
        editHappened => now;
        // update the gain and rate
        editRate => seqRate[editWhich];
        editGain => seqGain[editWhich];
        <<< num_vertices >>>;
        <<< "EDIT rate:", editRate, "gain:", editGain >>>;
    }
}
*/

// spork update
spork ~ playheadPosUpdate();
spork ~ set_and_run(local_N) @=> Shred @ s;

// this IS the control loop:
while(true)
{
    nChange => now;
    NUM_VERTICES => local_N;
    Machine.replace(s.id(), spork ~ set_and_run(local_N) @=> Shred @ s);
    
}